
//=====================================================
// Amaterske SQM - Sky Quality Meter
//   AstroMiK
//
//
// Detaily (schema, plosnak, navod, fotky):
//           http://sqm.astromik.org
//
# define verzeSW "2021-07-08"
//=====================================================



// volba typu senzoru teploty a vlhkosti (zvolit jen 1 ze 4 moznosti)
//#define senzor_DHT11                                   // DHT-11: nepresne cidlo teploty a vlhkosti
//#define senzor_DHT22                                   // DHT-22: trochu lepsi cidlo teploty a vlhkosti
#define senzor_BME                                       // BME280: presne cidlo teploty, tlaku a vlhkosti
//#define senzor_nic                                     // bez cidel (bude se merit jen plosny jas)


// volba zobrazeni (zvolit jen 1 ze 2 moznosti)
#define displej_TM1637_5                                 // LED displej 5x7 segmentu (TM1637)
//#define signal_LED                                     // misto displeje jen signalizacni LED


#define pouzivat_SD_kartu                                // zakomentovanim je mozne vypnout pouzivani SD karty (zustane jen EEPROM a seriova linka + pripadne displej)
                                                         // zrusi se tim varovna hlaseni o nezasunute karte pri startu programu a pri pokusu o zapis (zapisy budou uplne zrusene)

// vybrat jazyk
#define CZ_language                                      // cestina
//#define EN_language                                    // anglictina jeste neni prelozena (texty jsou v souboru "jazyky.h")     //!!


// nastaveni analogovych urovni pro test baterie (meri se za odporovym delicem R7, R8, R6)
//     merici rozsah je 0 az 3,3V a to odpovida cislu 0 az 4095.
//  Uvedene meze vychazi ze skutecne zjistenych hodnot - v priloze je dokument "test_baterie.xls"
#define bat_level_3     620                              // (7V) nad level_3 je vyborny stav baterie                        (blika   1 tecka na displeji)
#define bat_level_2     528                              // (6V) mezi level_3 a level_2 (7V az 6V)je dobry stav baterie     (blikaji 2 tecky na displeji)
#define bat_level_1     441                              // (5V) mezi level_2 a level_1 (6V az 5V) je stredni stav baterie  (blikaji 3 tecky na displeji)
                                                         // pod mezi level_1 (pod 5V)                                       (blikaji 4 tecky na displeji)
#define ref_level      2401                              // kdyz je napajeni tak nizke, ze referencni napeti prekroci tuto uroven, rozblika se vsech 5 tecek

// V minimalni variante se do EEPROM uklada techto 8 bajtu:
//     1. bajt       - znacka obsazeneho nebo volneho mista, znacka SEC/SELC, index prepnuteho cidla na rozsirujici desce
//     2. az 5. bajt - casovy udaj v sekundach od 1.1.1970
//     6. a 7. bajt  - udaj o namerenem plosnem jasu v tisicinach mag/arcsek2
//     8. bajt       - popis co zpusobilo spusteni mereni (tlacitko, seriova linka, automat, kalibrace), znacka stability mereneho jasu, adresa zarizeni pro komunikaci

// volitelne polozky, ktere se budou ukladat do EEPROM pri kazdem zaznamu (vice polozek znamena, ze se do EEPROM vejde mene zaznamu)
// --- PO ZMENE UKLADANYCH POLOZEK JE NUTNE PROVEST "HARD" FORMAT EEPROM (funkce "#FH" pres seriovou linku) ---
#define ukladat_teplotu             // do EEPROM se bude ukladat informace o teplote                                                           (plus  2 bajty na zaznam)
#define ukladat_tlak                // do EEPROM se bude ukladat informace o tlaku                                                             (plus  2 bajty na zaznam)
#define ukladat_vlhkost             // do EEPROM se bude ukladat informace o vlhkosti                                                          (plus  2 bajty na zaznam)
#define ukladat_infra               // do EEPROM se bude ukladat informace o INFRA slozce svetla                                               (plus  2 bajty na zaznam)
#define ukladat_full                // do EEPROM se bude ukladat informace o FULL slozce svetla                                                (plus  2 bajty na zaznam)
#define ukladat_contr_reg           // do EEPROM se bude ukladat informace o nastaveni Controll registru cidla svetla (zesileni, doba snimani) (plus  1  bajt na zaznam)
#define ukladat_GPS                 // do EEPROM se bude ukladat informace o zemepisnych souradnicich (nutny pridavny hardware)                (plus 10 bajtu na zaznam)

// priklad: Pro uplne ukladani vsech polozek bude 1 zaznam dlouhy 29 bajtu. Do 128kB EEPROM se tak vejde asi 4500 zaznamu.
//          Pro minimalni verzi ukladani (uklada se jen nezbytne minimum informaci a svetlo) je jeden zaznam dlouhy 8 bajtu a do stejne EEPROM se tak vejde pres 16000 zaznamu


byte delka_zaznamu;                                      // podle zvolenych polozek pro ukladani do EEPROM se nize vypocte delka jednoho zaznamu
unsigned long adr_dalsiho_zaznamu;                       // aby nebylo nutne pred kazdym ukladanim do EEPROM hledat volne misto, zapamatuje se do teto promenne adresa pristiho volneho prostoru
unsigned long adr_posl_zaznamu;                          // adresa, kam byl ulozen posledni zaznam (pro jednodussi ziskavani dat z EEPROM)

unsigned long max_zaznam_EEPROM;                         // adresa, na ktere je ulozeny prvni bajt z posledniho zaznamu (vypocitava se z velikosti pouzitelne EEPROM a z delky zaznamu)

#define min_EEPROM     500UL                             // Na jake adrese v EEPROM zacina oblast pro namerena data:
                                                         //     do adresy 20 jsou systemove parametry,
                                                         //     do adresy 50 je kalibracni tabulka pro teploty
                                                         //     od 50 do 500 jsou kalibracni tabulky pro cidla jasu (pro kazde ze 7 cidel zvlast)
#define max_EEPROM    0x1FFFFUL                          // maximalni mozna adresovatelna bunka v EEPROM  (pro 128k x 8b EEPROM )


#include <Wire.h>                                        // knihovna pro I2C komunikaci
#include <libmaple/iwdg.h>                               // podpora pro Watchdog
#include "jazyky.h"


#if defined senzor_DHT11 || defined senzor_DHT22
  #include <OneWireSTM.h>                                   // knihovna pro OneWire komunikaci (cidlo vlhkosti)
  #include <DHT.h>                                       // knihovna pro cidlo vlhkosti a teploty DHT11 / DHT22
  #define pin_DHT             PB14
#endif


#ifdef senzor_DHT11
    DHT cidlo_DHT(pin_DHT, DHT11);                       // inicializace DHT11
#endif
#ifdef senzor_DHT22
    DHT cidlo_DHT(pin_DHT, DHT22);                       // inicializace DHT22
#endif



#ifdef senzor_BME
  #include <Adafruit_BME280.h>                           // knihovna pro cidlo vlhkosti, teploty a tlaku
  Adafruit_BME280 cidlo_BME; // I2C
#endif


#include "RTClock.h"


RTClock rtclock (RTCSEL_LSE);
time_t tt;
time_t tt_file;
tm_t mtt;
tm_t mtt_file;
tm_t mtt_EEPROM;




#define pin_SD_CS           PA4                          // CS na SD karte

#define pin_tl_up          PA15                          // ovladaci tlacitka
#define pin_tl_dn           PB3
#define pin_tl_ok           PB4

#define pin_bzuk            PB1                          // pin pro piskak (pasivni)
#define pin_DERE           PB15                          // pin pro prepinani smeru vysilani 


#define pin_dis_CLK        PB12                          // piny pro komunikaci s displejem
#define pin_dis_DIN        PB13

#ifdef signal_LED
#define pin_LED_R          PA1                           // cervena LED pro verzi bez displeje
#define pin_LED_G          PB8                           // zelena LED pro verzi bez displeje
#define pin_LED_B          PB9                           // modra LED pro verzi bez displeje
#endif


#define pin_karta_IN       PB10                          // pin pro test zasunute karty ve druhe verzi plosnaku
#define pin_refu           PB0                           // pin pro referencni napeti pro kontrolu stavu baterie


#define pin_AD_BAT         PA0                           // pin pro testovani napeti baterie
#define pin_TS            PA14                           // specialni pin pro zaznam casoveho razitka (TimeStamp)

#define pin_TEST_HW       PA13                           // spusteni testu bez pripojene komunikace



#define pin_EXT_ZAP       PB11                           // pin pro zjistovani pripojeneho EXT zarizeni (GND = pripojeno)


#define I2C_ADDR_EEPROM_RW_0     0b1010000               // I2C adresa EEPROM pro cteni a zapis dat pro bunky 0 az 65535 (adresovaci piny v LOW)
#define I2C_ADDR_EEPROM_RW_1     0b1010001               // I2C adresa EEPROM pro cteni a zapis dat pro bunky nad 65535  (adresovaci piny v LOW)
byte I2C_ADDR;                                           // podle aktualni adresy zapisovane, nebo ctene bunky se nastavuje I2C adresa EEPROM

#define I2C_ADDR_EXPA    0x20                            // I2C adresa expanderu 0x20 (vsechny adresovaci piny v LOW)
#define I2C_ADDR_GPS     0x17                            // I2C adresa pridavneho modulu pro GPS


boolean leto =  false;                                   // letni cas

byte data;
boolean open_OK;                                         // pomocna promenna, jestli funguje zapis na kartu (jestli je zasunuta karta)

byte prumery;                                            // globalni promenna pro pocet mereni hodnot svetla, ze kterych se vypocte prumer
byte automat;                                            // globalni promenna pro nastaveni intervalu mezi automatickym merenim svetla v minutach (0=automat vypnuty)
byte selc;                                               // globalni promenna pro nastaveni casove zony (0=SEC, 1=SELC)
byte slave_addr;                                         // SLAVE adresa pro komunikaci                     
int list;                                                // listovani v poslednich zmerenych hodnotach
byte hranice_ukladani;                                   // hodnota svetla v desetinach mag/arcsek2 pod kterou se nebudou ukladat zaznamy do EEPROM ani na kartu

unsigned long posledni_autospusteni;                     // pomocna promenna pro cas posledniho automatickeho spusteni

char prijato[16];                                        // pole, do ktreho se ukladaji znaky, prijate pres seriovou linku (nastaveni casu, zadost o informace, korekce ....)
unsigned int i;                                          // pomocna globalni promenna pro ruzne smycky

byte prijimaci_pole[16];                                 // pole, do ktereho se ukladaji pozadavky od MASTERa pri komunikaci RS485
byte vysilaci_pole[35];                                  // pole, do ktereho se pripravuji data k odeslani do MASTERa pri komunikaci RS485

byte dokoncene_mereni = 2;                               // na zacatku mereni se tato promenna vynuluje a po dokonceni mereni (ktere muze trvat treba i 20 sekund) se nastvi zpatky na 2
byte prubeh_mereni;                                      // Kdyz prijde dotaz na posledni namerene hodnoty ( pres seriovou linku RS485), ale mereni jeste nebude ukoncene, odesle se zbyvajici pocet mereni

unsigned long casova_znacka;                             // pouzito pro zjisteni doby, jak dlouho se ma ignorovat komunikace pres RS485
unsigned int ignor_cas = 3000;

byte pouzij_CRC;                                         // podle parametru v EEPROM[15] je mozne vypnout kontrolu CRC pri prijmu pozadavku pres RS485
byte index_cidla_svetla = 1;                             // v pripade rozsirujici desky pro vice cidel je v teto promenne ulozene cislo prave aktivniho cidla 1 az 7 - (specialni pripad 0)
byte pocet_cidel_svetla;                                 // pocet cidel svetla na expanzni desce

byte lap_count;                                          // pocitadlo mezicasu od spusteni stopek (1 az 99)

boolean blokuj_TS;                                       // zaznam casove znacky je blokovan az do uvolneni tlacitka (aby pri drzeni tlacitka nebylo mozne neustale ukladani casovych znacek)

byte pole_EEPROM[35];                                    // globalni promenna, do ktere se pri kazdem mereni ukladaji namerena data. Toto pole se nasledne zapisuje do EEPROM a na SD kartu
byte pole_GPS_I2C[30];                                   // pole, do ktereho se stahuji data z GPS modulu pres I2C

unsigned int  int_svetlo;                                // globalni promenne, ktere uchovavaji posledni zmerene hodnoty
unsigned int  int_teplota;
unsigned int  int_vlhkost;
unsigned int  int_tlak;
unsigned int  int_infra;
unsigned int  int_full;
byte          byte_cnf;
unsigned long GPS_lat;
unsigned long GPS_lon;
unsigned int  GPS_alt;

byte posledni_i2C_prikaz;                                // aby se zbytecne neustale neobnovoval stav LED v GPS modulu

byte         znacka;                                     // znacka, co spustilo mereni (tlacitko, automat , seriova linka) + dalsi info (SLAVE adresa a stabilita svetla)

int pamet_LED_svitilna = 1;                              // az do vypnuti napajeni se v teto promenne uchovavanastaveny jas cervene svitilny (0...zhasnuto az 5...sviti vsech 5 sedmisegmentovek)

// promenne pro pripadnou korekci vnitrniho RTC
unsigned long cas_minuleho_nastaveni;                    // cas predchoziho nastaveni casu v sekundach od 1.1.1970
unsigned long korekce;                                   // po jake dobe se ma pricist nebo odecist 1 sekunda

byte rozhod_stab;                                        // rozhodovaci uroven pro stabilni jas
                      
byte D_pamet[5];                                         // V tomto poli se bitove uchovava aktualni stav vsech rozsvicenych segmentu
byte pozice_tecky;                                       // Pocet desetinnych mist pro zobrazeni cisla (0 az 4) - na pozici 0 ale tecka nikdy nesviti
long posledni_n;                                         // Pri aktualizaci pozice desetinne tecky se znovu zobrazi posledni zadane cislo 

byte pom_disp_scroll[30];                                // pro pripad, kdy je nutne scrollovat nejaky text na displeji, se ten text uklada sem do toho pole

int LOC_hod, LOC_den, LOC_mes, LOC_min, LOC_sek;         // promenne, ktere obsahuji aktualni cas v RTC (mistni cas)
unsigned int LOC_rok ;                                   // rok (od 1970)
byte LOC_dvt;                                            // den v tydnu (po ... ne)
unsigned long rokmesden;                                 // rok mesic a den ve formatu yyyymmdd
unsigned int mesden;                                     // mesic a den ve formatu mmdd


int LOC_den_file, LOC_mes_file;                          // promenne, ktere obsahuji datum pro nazev slozky na SD karte
unsigned int LOC_rok_file ;                              // rok (od 1970)


#ifdef displej_TM1637_5           // definice znaku pro displej se spolencou anodou
                  //       0    ,       1    ,       2    ,       3    ,       4    ,       5    ,       6    ,       7    ,       8    ,       9    
byte graf_def[10] = {0b00111111 , 0b00000110 , 0b01011011 , 0b01001111 , 0b01100110 , 0b01101101 , 0b01111101 , 0b00000111 , 0b01111111 , 0b01101111 };
#endif

byte pole_citlivosti[4] = {0 , 16 , 32 , 48 };           // stupen citlivosti cidla
int posledni_citlivost = 3;                              // posledni pouzity stupen citlivosti (pouziva se k uychleni mereni svetla pri vice merenich v rade)


unsigned long pole_baudrate[4] = {9600 , 19200 , 38400 , 115200 };      // prednastavene komunikacni rychlosti


byte znacka_listu;                                       // pri vypisu svetla se jeste pred datumem vypise informace o typu zaznamu:
                                                         //   'c' = jeden z 10 kalibracnich vzorku
                                                         //   'C' = prumer vsech 10 kalibracnich vzorku
                                                         //   'u' = spusteno tlacitkem "Up"
                                                         //   'd' = spusteno tlacitkem "Down"
                                                         //   'o' = spusteno tlacitkem "Ok"
                                                         //   'A' = spusteno automatem
                                                         //   'M' = spusteno komunikaci


byte znak_stupen = 32;

int kal_svetlo[20][2] ;                                  // pole kalibracnich hodnot pro svetlo
int kal_teplota[10][2];                                  // pole kalibracnich hodnot pro teplotu
int kal_serial[20][2] ;                                  // pres seriovou linku se nejdriv ulozi prijata data do tohoto pole. Po jejich prekontrolovani se pak prepisou do jednoho z poli kal_teplota[], nebo kal_svetlo[]


//  index pole 100x                                                                                                            1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111222222222222222222222     
//  index pole 10x                   1111111111222222222233333333334444444444555555555566666666667777777777888888888899999999990000000000111111111122222222223333333333444444444455555555556666666666777777777788888888889999999999000000000011111111112    
//  index pole 1x          01234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890    
//                         ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
char vystupni_retezec[] = "                                                                                                                                                                                                                           \0";
// priklad                " 00018 ; 000823 ; 1609232363 ; 29.12.2020 ; 08:59:23 ; Ut ;  SEC ; 08881 ; 08,881 ; 01 ; 01 ; komun ;  STABIL  ; 17989 ; 48311 ; 600 ;  428x ; 07196 ; +21,96 ; 05151 ; 051,51 ; 093410 ; +49,457154 ; +120,457156 ; +1234 ";    



unsigned long stisk_time;                                // pro pocitani delky stisku tlacitka
boolean stisk;                                           // pro pociteni delky stisku tlacitka
boolean bude_save;                                       // pri opusteni menu tato promenna urcuje, jestli se ma nova hodnota ulozit do EEPROM/(RTC), nebo jestli se nic ukladat nebude 
byte jas_displeje;                                       // globalni promenna pro nastaveni jasu displeje podle posledni zmerene hodnoty svetla

byte spec_rezim;                                         // podle pripojenych I2C periferii je mozne provozovat zarizeni v ruznych specialnich rezimech

byte oddelovaci_bajt;                                    // obsah EEPROM(9), ktery obsahuje informace o oddelovacich znaminkach pro CSV soubor na SD karte
boolean des_carka;                                       // oddelovac desetinnych mist (tecka = false; carka = true)
boolean uzavirat_uvozovky;                               // uzavirat kazdou hodnotu na SD karte do uvozovek
boolean vytvaret_hlavicku;                               // Na zacatek kazdeho souboru na SD karte zapise (nebo nezapise) textovy popis sloupcu
byte oddelovac_polozek;                                  // Urcuje znak, kterym se budou oddelovat jednotlive polozky v CSV souboru (mezera, strednik, carka, tabulator)

char oddelovac;                                          // znak pro oddelovani polozek v zaznamech (strednik, carka, mezera, tabulator)
char uvozovky;                                           // znak pro uzavirani polozek do uvozovek (nebo mezera)
char desod;                                              // znak pro oddeleni desetinnych mist (carka, nebo tecka)

#ifdef pouzivat_SD_kartu                                 // kdyz je nastaveno pouzivani SD karty, tak se muze ukladat

#include "SdFat.h"                                       // nova knihovna pro praci s SD kartou
#include "sdios.h"

SdFat sd;                                                // pro SD kartu
SdFile soubor;
#endif



//======================================================================================================================
void setup(void)
  {

    afio_cfg_debug_ports(AFIO_DEBUG_NONE);         // zprovozneni JTAG pinu jako obycejnych GPIO (PB3 = tlacitko "DOLU"; SWSCK = TimeStamp; SWDIO = HW test baterie )
    
    pinMode(pin_SD_CS, OUTPUT);

    pinMode(pin_tl_up, INPUT_PULLUP);
    pinMode(pin_tl_dn, INPUT_PULLUP);
    pinMode(pin_tl_ok, INPUT_PULLUP);

    pinMode(pin_EXT_ZAP, INPUT_PULLUP);


    pinMode(pin_bzuk,OUTPUT);

    pinMode(pin_dis_CLK, OUTPUT);
    pinMode(pin_dis_DIN, OUTPUT);
    pinMode(pin_DERE, OUTPUT);





#ifdef signal_LED
    pinMode(pin_LED_R, OUTPUT);
    analogWrite(pin_LED_R , 255);                  // LED ma spolecnou anodu, takze se zhasina pomoci 100% PWM
    pinMode(pin_LED_G, OUTPUT);
    analogWrite(pin_LED_G , 255);                  // LED ma spolecnou anodu, takze se zhasina pomoci 100% PWM
    pinMode(pin_LED_B, OUTPUT);
    analogWrite(pin_LED_B , 255);                  // LED ma spolecnou anodu, takze se zhasina pomoci 100% PWM
#endif  

 
    digitalWrite(pin_dis_CLK , LOW);
    digitalWrite(pin_dis_DIN , LOW);


    pinMode(pin_karta_IN, INPUT_PULLUP);
    pinMode(pin_AD_BAT  , INPUT_ANALOG);
    pinMode(pin_TEST_HW , INPUT_PULLDOWN);         // pin pro specialni testy urovni napajeciho napeti (kdyz nesmi byt zapojena komunikace)
    pinMode(pin_refu    , INPUT_ANALOG);
    pinMode(pin_TS      , INPUT_PULLUP);
    

    Wire.begin();
    delay(100);

    jas_displeje = 3;



    // po resetu je mozne obnovit defaultni nastaveni promennych v EEPROM (bez kalibrace) dlouhym stiskem vsech 3 tlacitek zaroven
    if (digitalRead(pin_tl_up) == LOW and digitalRead(pin_tl_dn) == LOW and digitalRead(pin_tl_ok) == LOW) 
      {
        tone(pin_bzuk,4000, 10);
        unsigned long starttime = millis();
        while (digitalRead(pin_tl_up) == LOW and digitalRead(pin_tl_dn) == LOW and digitalRead(pin_tl_ok) == LOW)
          {
            delay(50);
            if (millis() - starttime > 5000UL)
              {
                tone(pin_bzuk,4000, 10);
                delay(100);
                default_parametry();

#ifdef displej_TM1637_5
                zobraz_text(20);                   // dEFA
                delay(500);                
                zobraz_text(28);                   // prazdny displej
#endif                
#ifdef signal_LED
                LED_R_onoff(true);
                LED_G_onoff(true);
                LED_B_onoff(true);
                delay(500);                
                LED_R_onoff(false);
                LED_G_onoff(false);
                LED_B_onoff(false);
                
#endif
                
                break;
              }
          }
        while (digitalRead(pin_tl_up) == LOW or digitalRead(pin_tl_dn) == LOW or digitalRead(pin_tl_ok) == LOW)
          {
            tone(pin_bzuk,3000, 10);
            delay(500);
          }
        delay(100);

        delay(1000);
        SOFT_RESET();
      
      }


    prijem();

    

#ifdef pouzivat_SD_kartu                           // kdyz je nastaveno pouzivani SD karty, ale karta neni zasunuta, zobrazi se pri startu na sekundu varovani, nebo blikne modra LED
    if (digitalRead(pin_karta_IN)== HIGH)          // karta vysunuta
      {
        chyba(3);                                  // No-SD
      }
#endif

#ifdef displej_TM1637_5
    zobraz_text(28);                               // zhasne displej pro pripad, ze by neco zustalo v jeho pametech
#endif

#ifdef signal_LED                  
    LED_B_onoff(false);                            // zhasne modrou LED
#endif

// vypocet delky zaznamu podle zvolenych polozek pro ukladani do EEPROM
    delka_zaznamu = 8;
#ifdef ukladat_teplotu    
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_tlak       
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_vlhkost    
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_infra      
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_full       
    delka_zaznamu = delka_zaznamu + 2;
#endif
#ifdef ukladat_contr_reg 
    delka_zaznamu = delka_zaznamu + 1;
#endif

#ifdef ukladat_GPS 
    delka_zaznamu = delka_zaznamu + 10;
#endif


    // vypocet adresy prvniho bajtu posledniho zaznamu
    max_zaznam_EEPROM = min_EEPROM + ((max_EEPROM - min_EEPROM - delka_zaznamu) / delka_zaznamu) * delka_zaznamu;

    byte zachytny_bod_hledani = EEPROM_read(18);
    adr_dalsiho_zaznamu = najdi_prazdne_misto(zachytny_bod_hledani);      // pri zapnuti se musi prohledat cela pamet od posledniho zachytneho bodu, ale priste uz se bude brat prazdny prostor z hodnoty teto promenne


#ifdef senzor_BME
    cidlo_BME.begin(0x76);
#endif  

#if defined senzor_DHT11 || senzor_DHT22
    cidlo_DHT.begin();
#endif

   
    // nastaveni komunikacni rychlosti podle bitu 4 a 5 na adrese 14 v EEPROM
    Serial.begin(pole_baudrate[(EEPROM_read(14) & 0b00110000) >> 4]);           // servisni a nahravaci UART (piny PA9, PA10) 
    Serial2.begin(pole_baudrate[(EEPROM_read(14) & 0b00110000) >> 4]);          // RS485  (piny PA2, PA3)
    
    // aby se nemuselo pri kazdem pozadavku o cas z RTC  sahat do EEPROM, nastavi se nasledujici globalni promenne jen pri startu, nebo pri dalsim serizeni RTC
    cas_minuleho_nastaveni = EEPROM_read_long(0);             // Z EEPROM se zjisti, kdy se naposled serizoval cas (cas je v sekundach od 1.1.1970)
    korekce                = EEPROM_read_long(4);             // a jaka byla pri te prilezitosti vypoctena korekce (za jak dlouho se pricist, nebo odecist extra sekunda)


    automat = EEPROM_read(10);                     // v EEPROM se na adrese 10 nachazi cislo, ktere udava, v jakych intervalech (minuty) se ma merit jas povoleny rozsah 0 az 255 (0=automat vypnuty)

    prumery = EEPROM_read(11);                     // v EEPROM se na adrese 11 nachazi cislo, ktere udava, kolik se ma prumerovat hodnot povoleny rozsah 1 az 20
    if (prumery == 0 or prumery > 20)              // kdyz je tam nejaky nesmysl, tak se provede kompletni prepsani defaultnich parametru systemu do EEPROM
      {
        default_parametry();
        delay(1000);
        SOFT_RESET();
      }

    hranice_ukladani = EEPROM_read(19);            // od jake urovne svetla se maji ukladat zaznamy do EEPROM

      
    slave_addr = (0b00001111 & EEPROM_read(14));   // v EEPROM se na adrese 14 nachazi SLAVE adresa pro komunikaci (bity 0 az 3). Na bitech 4 a 5 je komunikacni rychlost.
    if (slave_addr == 0 or slave_addr > 15)        // kdyz je tam nejaky nesmysl, tak se prepise na 1
      {
        slave_addr = 1;
        EEPROM_write(14,1);                        // komunikacni rychlost se v tom pripade take zmeni na defaultni 9600
      }



    pouzij_CRC = EEPROM_read(15);

    rozhod_stab = EEPROM_read(16) ;

    pocet_cidel_svetla = EEPROM_read(17) ;
    
    
    selc = EEPROM_read(8);                         // v EEPROM se na adrese 8 nachazi casova zona (0=SEC, 1 = SELC)
    if (selc > 1) 
      {
        selc = 1;
        EEPROM_write(8,selc);
      }
    if (selc == 0) leto = false;
    else           leto = true;

    oddelovaci_bajt = EEPROM_read(9);
    des_carka         = bitRead(oddelovaci_bajt,2);
    uzavirat_uvozovky = bitRead(oddelovaci_bajt,3);
    vytvaret_hlavicku = bitRead(oddelovaci_bajt,4);
    oddelovac_polozek = oddelovaci_bajt & 0b00000011;

    if (des_carka == true)  desod = ',';
    else                    desod = '.';

    if (uzavirat_uvozovky == true)  uvozovky = '"';
    else                            uvozovky = ' ';     

    
    // nacteni kalibracnich dat pro svetlo z EEPROM pro cidlo s indexem 1
    for (byte i = 0 ; i < 15 ; i++)
      {
        kal_svetlo[i][0] = EEPROM_read_int((i*4) + 50);
        kal_svetlo[i][1] = EEPROM_read_int((i*4) + 52);
      }

    // nasledujici 2 kalibracni hodnoty jsou napevno a neukladaji se do EEPROM
    kal_svetlo[15][0] = 0;                         //  [15]   - tohle musi byt minimalni zmeritelna hodnota svetla (nejvetsi svetlo) (0 mag)  (nic mensiho se nesmi namerit)
    kal_svetlo[15][1] = 0;
    kal_svetlo[16][0] = 24000;                     //  [16]   - tohle musi byt maximalni zmeritelna hodnota svetla (nejvetsi tma) (24,000 mag)(nic vetsiho se nesmi namerit)
    kal_svetlo[16][1] = 24000;

    test_korekcni_tabulky_svetla();                // zkontroluje spravne zadane udaje



    // nacteni kalibracnich dat pro teplotu z EEPROM
    for (byte i = 0 ; i < 7 ; i++)
      {
        kal_teplota[i][0] = EEPROM_read_int((i*4) + 20);
        kal_teplota[i][1] = EEPROM_read_int((i*4) + 22);
      }
    // nasledujici 2 kalibracni hodnoty jsou napevno a neukladaji se do EEPROM
    kal_teplota[7][0] = 0;                         //  [7]   - tohle musi byt minimalni zmeritelna hodnota teploty (-50'C)(nic mensiho se nesmi namerit)
    kal_teplota[7][1] = 0;
    kal_teplota[8][0] = 10000;                     //  [8]   - tohle musi byt maximalni zmeritelna hodnota teploty (+50'C) (nic vetsiho se nesmi namerit)
    kal_teplota[8][1] = 10000;

    test_korekcni_tabulky_teploty();               // zkontroluje spravne zadane udaje


    // vytvoreni unikatniho identifikacniho cisla - provede se jen pri prvnim spusteni kdyz je EEPROM vynulovana
    if (EEPROM_read_int(12) == 0 or EEPROM_read_int(12) == 65535)
      {
        EEPROM_write_int(12 , svetlo_1x(false, true));     // mikroluxy jsou skoro nahodne cislo
        
      }


    prepni_cidlo_svetla(1);                        // v pripade pouziti rozsirujici desky pro vice cidel svetla se prepne na to prvni.
                                                   //   kdyz deska neexistuje, zustane jen jedno cidlo na zakladni desce

    // test funkcnosti RTC (pocet sekund od 1.1.1970 musi byt vetsi, nez interval do 1.1.2020)
    unsigned long tt = rtclock.now();
    if (tt < 1577833000UL)                         // datum v RTC je drivejsi nez rok 2020
      {
        chyba(4);                                  // chyba RTC
      }



    // Do funkce 'STOPKY' se da vstoupit hned po zapnuti napajeni a soucasnem stisku tlacitka OK
    if (digitalRead(pin_tl_up) == HIGH and digitalRead(pin_tl_dn) == HIGH and digitalRead(pin_tl_ok) == LOW) 
      {
        stopky();
      }


    if (digitalRead(pin_TS) == LOW)                // pri zkratu pinu SCK a GND na bocnim konektoru desky WeAct (BluePill +) se zaznamena casove razitko
      {
        casove_razitko();
        while(digitalRead(pin_TS) == LOW)          // ceka se na uvolneni tlacitka (vcetne odruseni zakmitu kontaktu)
          {
            delay(50);
          }
        delay(50);
      }
    
    
    if (automat > 0)                               // kdyz je nastavene automaticke snimani
      {
        posledni_autospusteni = millis();
        mereni(6);                                 //  tak se prvni mereni provede hned po zapnuti napajeni
      }  
  }
//======================================================================================================================



//======================================================================================================================
void loop(void)
  {

    test_baterie(0);

    if(digitalRead(pin_TEST_HW) == HIGH)                   // jen testy A/D prevodniku pro vyhodnoceni stavu baterie - bez pripojene komunikace
      {
        zobraz_text(34);                                   // "rEF_U"
        delay(3800);        
        zobraz_cislo(analogRead(pin_refu),0);
        delay(2500);

        zobraz_text(35);                                   // "U_bAt"
        delay(3800);        
        zobraz_cislo(analogRead(pin_AD_BAT),0);
        delay(2500);    
      }
    
    interrupty();                                          // odskoceni na test, jestli neni nejaky pozadavek o komunikaci, nebo jestli neni cas spustit automaticke mereni


    if (digitalRead(pin_tl_up) == LOW)                     // zaznam hodnoty s tlacitkem "NAHORU"
      {
        delay(20);
        mereni(1);                                         // parametr je znacka tlacitka, ktera se uklada do zaznamu
        while (digitalRead(pin_tl_up) == LOW) delay(20);   // cekani na uvolneni tlacitka
        delay (20);                                        // odruseni zakmitu pri uvolneni tlacitka "NAHORU"
      }


    if (digitalRead(pin_tl_dn) == LOW)                     // zaznam hodnoty s tlacitkem "DOLU"
      {
        delay(20);
        mereni(2);                                         // parametr je znacka tlacitka, ktera se uklada do zaznamu
        while (digitalRead(pin_tl_dn) == LOW) delay(20);   // cekani na uvolneni tlacitka
        delay (20);                                        // odruseni zakmitu pri uvolneni tlacitka "NAHORU"
      }


    if (digitalRead(pin_tl_ok) == LOW)                     // stisk tlacitka OK (tady se rozlisuje kratky a dlouhy stisk)
      {
        stisk_time = millis();
        stisk = true;
        delay(20);                                         // odruseni zakmitu pri stisku tlacitka
        unsigned long trvani;
        while (digitalRead(pin_tl_ok) == LOW)
          {
            delay(20);                                     // cekani na uvolneni tlacitka
            trvani = millis() - stisk_time;
            if (trvani > 500)                              // dlouhy stisk = vstup do menu
              {
                
#ifdef signal_LED
                LED_R_onoff(true);
                menu_LED();                                // vstup do LED menu (jen zakladni polozky, ktere jsou signalizovany blikanim LED )
#else
                menu_1();                                  // vstup do menu se zobrazenim polozek na nekolika 7-segmentovych displejich
#endif


              }
        
          }
        if (trvani < 500)                                  // kratky stisk = mereni
          {
            mereni(3);                                     // pro tlacitko OK se provadi mereni az pri uvolneni
          }       

        stisk=false;
      }

    delay(100);
  
  }
//======================================================================================================================



//------------------------------------------
// systemove informace
void sys_info(void)
  {
    unsigned int pomprom;
    
    Serial.print("SW ver.: ");                                 //             verze software
    Serial.println(verzeSW);                                   //             (datum nebo jiny text nadefinovany na zacatku programu)
    Serial.println(' ');                                       //             prazdna radka

    
    pomprom = EEPROM_read(8);
    Serial.print(lng111);                                      //             Serial.print("Casova zona: ");
    if (pomprom == 0)Serial.println(lng006);                   //             Serial.println("SEC.");
    else             Serial.println(lng007);                   //             Serial.println("SELC.");

    pomprom = EEPROM_read(9);
    Serial.println(lng112);                                    //             Serial.println("\nOddelovace:");
    
    Serial.print(lng113);                                      //             Serial.print(" Polozky v souboru: ");
    if ((pomprom & 0b11) ==     0 )  Serial.println(lng078);   //             Serial.println("mezera");
    if ((pomprom & 0b11) ==     1 )  Serial.println(lng079);   //             Serial.println("carka");
    if ((pomprom & 0b11) ==     2 )  Serial.println(lng080);   //             Serial.println("strednik");
    if ((pomprom & 0b11) ==     3 )  Serial.println(lng081);   //             Serial.println("tabulator");

    Serial.print(lng114);                                      //             Serial.print(" Desetinny oddelovac: ");
    if ((pomprom & 0b100) ==    4 )  Serial.println(lng079);   //             Serial.println("carka");
    else                             Serial.println(lng083);   //             Serial.println("tecka");

    if ((pomprom & 0b1000) ==   8 )  Serial.println(lng115);   //             Serial.println(" Uzavirat polozky do uvozovek");
    else                             Serial.println(lng116);   //             Serial.println(" Neuzavirat polozky do uvozovek");

    if ((pomprom & 0b10000) == 16 )  Serial.println(lng117);   //             Serial.println(" Do souboru vkladat hlavicky");
    else                             Serial.println(lng118);   //             Serial.println(" Hlavicky do souboru nevkladat");

    Serial.print(lng119);                                      //             Serial.println("\nAutomat:");        
    pomprom = EEPROM_read(10);
    if (pomprom == 0)                Serial.println(lng120);   //             Serial.println(" Bez automatickeho mereni");
    else
      {
                                     Serial.print(lng121);     //             Serial.print("  Spoustet mereni kazdych ");
                                     Serial.print(pomprom);
                                     Serial.println(lng122);   //             Serial.println(" minut");
      }

    pomprom = EEPROM_read(11);
    Serial.print(lng123);                                      //             Serial.print("\nPrumerovat ");
    Serial.print(pomprom);
    Serial.println(lng124);                                    //             Serial.println(" mereni plosneho jasu");

    pomprom = EEPROM_read(15);
    if (pomprom == 0)  Serial.println(lng125);                 //             Serial.println("Kontrolni bajt se pri komunikaci pres RS485 nevyhodnocuje.");
    else               Serial.println(lng126);                 //             Serial.println("Pri komunikaci pres RS485 je nutne odesilat spravny kontrolni bajt.");

    pomprom = EEPROM_read(16);
    Serial.print(lng127);                                      //             Serial.print("Rozhodovaci rozdil pro stabilni/nestabilni jas je ");
    Serial.print(pomprom/10.0,1);
    Serial.println(" %");

    pomprom = EEPROM_read(19);
    if (pomprom > 0)
      {
        Serial.print(lng128);                                  //             Serial.print("Do EEPROM se ukladaji zaznamy s plosnym jasem nad (tmavsi nez) ");
        Serial.print(pomprom/10.0);
        Serial.println(" mag/arcsek2");
      }
    else
      {
        Serial.println(lng207);                                //             Serial.print("Do EEPROM se ukladaji vsechny zaznamy");        
      }
   
    pomprom = EEPROM_read_int(12);
    Serial.print(lng129);                                      //             Serial.print("Unikatni identifikace EEPROM: ");
    Serial.print(" 0x");
    Serial.println(pomprom,HEX);

    pomprom = (0b00001111 & EEPROM_read(14));
    Serial.print(lng019);                                      //             Serial.print("SLAVE adresa: ");
    Serial.println(pomprom);



    Serial.println(lng130);                                    //             Serial.println("\nHardware:");

    Serial.println(lng131);                                    //             Serial.println(" Procesor: STM32F103CB (BluePill) - 128kB progMEM");    
    Serial.println(lng132);                                    //             Serial.println(" Cidlo jasu: TSL2591");    
#ifdef senzor_DHT11
    Serial.println(lng133);                                    //             Serial.println(" Cidlo vlhkosti a teploty: DHT11");
#endif
#ifdef senzor_DHT22
    Serial.println(lng134);                                    //             Serial.println(" Cidlo vlhkosti a teploty: DHT22");
#endif
#ifdef senzor_BME
    Serial.println(lng135);                                    //             Serial.println(" Cidlo vlhkosti, teploty a tlaku: BME280");
#endif

    Serial.print(lng136);                                      //             Serial.print(" SD karta: ");
#ifdef pouzivat_SD_kartu
    if (digitalRead(pin_karta_IN) == LOW) Serial.println(lng137);//           Serial.println("zasunuta");
    else                                  Serial.println(lng138);//           Serial.println("vysunuta");
#else
    Serial.println(lng139);                                    //             Serial.println("nepouziva se");
#endif
    
#ifdef displej_TM1637_5
    Serial.println(lng140);                                    //             Serial.println(" Displej: 5x7 segmentu (TM1637)");
#endif

#ifdef signal_LED
    Serial.println(lng141);                                    //             Serial.println(" Pouze signalizacni LED");
#endif

    if (pocet_cidel_svetla > 1)          // kdyz neni nastavena expanzni deska pro cidla svetla, nema cenu vypisovat hlaseni o dostupnych vstupech
      {
        Serial.print(lng142);                                  //             Serial.print(" Pocet vstupu rozsirujici desky: ");
        Serial.println(pocet_cidel_svetla);
        Serial.print(lng143);                                  //             Serial.print(" Aktualne aktivni cidlo jasu:    ");
        Serial.println(index_cidla_svetla);
      }


    Serial.println(lng144);                                    //             Serial.println("\nEEPROM:");
    Serial.print(lng145);                                      //             Serial.print(" Zacatek oblasti pro data: ");
    dopln_mezery(min_EEPROM,6);
    Serial.println(min_EEPROM);
    
    Serial.print(lng146);                                      //             Serial.print(" Konec oblasti pro data:   ");
    dopln_mezery(max_zaznam_EEPROM + delka_zaznamu - 1 ,6);
    Serial.println(max_zaznam_EEPROM + delka_zaznamu - 1);

    Serial.print(lng147);                                      //             Serial.print(" Pristi zaznam se ulozi na adresu: ");
    Serial.println(adr_dalsiho_zaznamu);


    Serial.print(lng148);                                      //             Serial.print(" Jeden zaznam v EEPROM je dlouhy: ");
    Serial.print(delka_zaznamu);
    Serial.println(lng149);                                    //             Serial.println (" bajtu");

    Serial.println(lng150);                                    //             Serial.println(" obsahuje tyto polozky: ");
    Serial.println(lng151);                                    //             Serial.println("  - plosny jas           ANO");

    Serial.print(lng152);                                      //             Serial.print  ("  - teplota              ");
#ifdef ukladat_teplotu
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng155);                                      //             Serial.print  ("  - tlak                 ");
#ifdef ukladat_tlak
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng156);                                      //             Serial.print  ("  - vlhkost              ");
#ifdef ukladat_vlhkost
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng157);                                      //             Serial.print  ("  - infra slozka svetla  ");
#ifdef ukladat_infra
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng158);                                      //             Serial.print  ("  - full svetlo          ");
#ifdef ukladat_full
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng159);                                      //             Serial.print  ("  - ctrl registr TSL2591 ");        
#ifdef ukladat_contr_reg
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif

    Serial.print(lng206);                                      //             Serial.print  ("  - GPS souradnice       ");        
#ifdef ukladat_GPS
    Serial.println(lng153);                                    //             Serial.println("ANO");
#else
    Serial.println(lng154);                                    //             Serial.println("NE");
#endif
    
    Serial.println(lng160);                                    //             Serial.println("\nKalibrace cidla teploty");
    Serial.println(lng161);                                    //             Serial.println("index ; merena  ; skutecna");
    Serial.println("-----------------------------");
    for (byte i = 0 ; i < 7 ; i++)
      {
        pomprom = EEPROM_read_int((i*4)+20);
        Serial.print(i+1);
        Serial.print(" ; ");
        dopln_mezery(citelna_teplota(pomprom),5);
        Serial.print( citelna_teplota(pomprom),2);             // (citelna merena teplota)
        Serial.print(" 'C ; ");

        pomprom = EEPROM_read_int((i*4)+22);
        dopln_mezery(citelna_teplota(pomprom),5);
        Serial.print( citelna_teplota(pomprom),2);             // (citelna zobrazovana teplota)
        Serial.println(" 'C");        
      }


    Serial.print(lng162);                                      //             Serial.print("\nKalibrace cidla plosneho jasu [");
    Serial.print(index_cidla_svetla);
    Serial.println("]");
    Serial.println(lng163);                                    //             Serial.println("index ; merene ; skutecne");
    Serial.println("--------------------------");
    unsigned int posun = ((index_cidla_svetla-1) * 60) + 50; 
    for (byte i = 0 ; i < 15 ; i++)
      {
        if( i < 9)   Serial.print(' ');                        // jen zarovnani cisel polozek
        Serial.print(i+1);
        Serial.print(" ; ");
        pomprom = EEPROM_read_int(posun + (i*4));
        dopln_mezery(citelne_svetlo(pomprom),5);
        Serial.print(citelne_svetlo(pomprom),3);
        Serial.print(" ; ");
        pomprom = EEPROM_read_int(posun + (i*4) + 2);
        dopln_mezery(citelne_svetlo(pomprom),5);
        Serial.println(citelne_svetlo(pomprom),3);
      }    
  
  }
//----------------------------------------------



//---------------------------------------------------------------------
// formatovani vypisu kalibracnich konstant
//   cislo se hned na zacatku oseka o desetinne casti
//  'pocet_mist' udava, na jak dlouhy retezec se doplni cela cast cisla
//   priklady parametru:    15.354 , 4     prida pred retezec 2 mezery
//                          1.078  , 4     prida pred retezec 3 mezery
//                         -24.11  , 3     neprida zadnou mezeru
void dopln_mezery(int cislo, byte pocet_mist)
  {
    byte delka_cisla = 5;
    if (cislo <     0)   delka_cisla ++; // zaporne cislo pridava jeden znak pro znamenko
    cislo = abs(cislo);
    if (cislo < 10000)   delka_cisla --;
    if (cislo <  1000)   delka_cisla --;
    if (cislo <   100)   delka_cisla --;
    if (cislo <    10)   delka_cisla --;

    while (delka_cisla < pocet_mist)
      {
        delka_cisla ++;
        Serial.print(' ');
      }
    
  }
//----------------------------------------------



//----------------------------------------------
void default_kalibrace_svetla(void)
  {

    for (byte i = 1; i < 8 ; i++)
      {
        // v EEPROM se nachazi 7 bloku kalibracnich dat (pro 7 cidel na rozsirujici desce), kazdy blok ma 15 bodu, kazdy bod ma 2 + 2 bajty (merena a skutecna hodnota) = 60 bajtu na blok
        //               bloky zacinaji na adrese 50 (za kalibracni tabulkou teplot)
        // Nasledujici zapis nulovych hodnot by se dal udelat jednoduchou smyckou, ale pro pripad, ze by se ukazalo, ze je lepsi zapisovat nejake jine hodnoty jsem to nechal oddelene
        
        unsigned int posun = ((i-1) * 60) + 50;   
        EEPROM_write_int( 0 + posun   ,     0);  //  1. kalibracni bod pro svetlo -  index[0]
        EEPROM_write_int( 2 + posun   ,     0);
    
        EEPROM_write_int( 4 + posun   ,     0);  //  2. kalibracni bod pro svetlo -  index[1]
        EEPROM_write_int( 6 + posun   ,     0);
    
        EEPROM_write_int( 8 + posun   ,     0);  //  3. kalibracni bod pro svetlo -  index[2]
        EEPROM_write_int(10 + posun   ,     0);
    
        EEPROM_write_int(12 + posun   ,     0);  //  4. kalibracni bod pro svetlo -  index[3]
        EEPROM_write_int(14 + posun   ,     0);   
    
        EEPROM_write_int(16 + posun   ,     0);  //  5. kalibracni bod pro svetlo -  index[4]
        EEPROM_write_int(18 + posun   ,     0);
    
        EEPROM_write_int(20 + posun   ,     0);  //  6. kalibracni bod pro svetlo -  index[5]
        EEPROM_write_int(22 + posun   ,     0);
    
        EEPROM_write_int(24 + posun   ,     0);  //  7. kalibracni bod pro svetlo -  index[6]
        EEPROM_write_int(26 + posun   ,     0);
    
        EEPROM_write_int(28 + posun   ,     0);  //  8. kalibracni bod pro svetlo -  index[7]
        EEPROM_write_int(30 + posun   ,     0);
    
        EEPROM_write_int(32 + posun   ,     0);  //  9. kalibracni bod pro svetlo -  index[8]
        EEPROM_write_int(34 + posun   ,     0);
    
        EEPROM_write_int(36 + posun   ,     0);  // 10. kalibracni bod pro svetlo -  index[9]
        EEPROM_write_int(38 + posun   ,     0);
    
        EEPROM_write_int(40 + posun   ,     0);  // 11. kalibracni bod pro svetlo -  index[10]
        EEPROM_write_int(42 + posun   ,     0);
    
        EEPROM_write_int(44 + posun   ,     0);  // 12. kalibracni bod pro svetlo -  index[11]
        EEPROM_write_int(46 + posun   ,     0);
    
        EEPROM_write_int(48 + posun   ,     0);  // 13. kalibracni bod pro svetlo -  index[12]
        EEPROM_write_int(50 + posun   ,     0);
        
        EEPROM_write_int(52 + posun   ,     0);  // 14. kalibracni bod pro svetlo -  index[13]
        EEPROM_write_int(54 + posun   ,     0);
    
        EEPROM_write_int(56 + posun   ,     0);  // 15. kalibracni bod pro svetlo -  index[14]
        EEPROM_write_int(58 + posun   ,     0);
        
      }
  }
//----------------------------------------------



//----------------------------------------------
void default_kalibrace_teploty(void)
  {
    // Nasledujici zapis hodnot 0 by se dal udelat jednoduchou smyckou, ale pro pripad, ze by se ukazalo, ze je lepsi zapisovat nejake jine hodnoty jsem to nechal oddelene
    EEPROM_write_int(20   ,  0);  // 1. kalibracni bod pro teplotu -  index[0]   
    EEPROM_write_int(22   ,  0);

    EEPROM_write_int(24   ,  0);  // 2. kalibracni bod pro teplotu -  index[1]   
    EEPROM_write_int(26   ,  0);

    EEPROM_write_int(28   ,  0);  // 3. kalibracni bod pro teplotu -  index[2]   
    EEPROM_write_int(30   ,  0);

    EEPROM_write_int(32   ,  0);  // 4. kalibracni bod pro teplotu -  index[3]   
    EEPROM_write_int(34   ,  0);

    EEPROM_write_int(36   ,  0);  // 5. kalibracni bod pro teplotu -  index[4]   
    EEPROM_write_int(38   ,  0);

    EEPROM_write_int(40   ,  0);  // 6. kalibracni bod pro teplotu -  index[5]   
    EEPROM_write_int(42   ,  0); 

    EEPROM_write_int(44   ,  0);  // 7. kalibracni bod pro teplotu -  index[6]   
    EEPROM_write_int(46   ,  0); 
  }

void default_parametry(void)
  {
     EEPROM_write_long(    0   ,           0);       // cas posledniho serizeni = 1.1.1970 0:00:00
     EEPROM_write_long(    4   ,  0x7FFFFFFFUL);     // sekunda se pricita jednou za 68 let
     EEPROM_write(         8   ,           0);       // SEC
     EEPROM_write(         9   ,  0b00010110);       // des. oddelovac carka, oddelovac polozek strednik, hlavicku vkladat, nezavirat do uvozovek
     EEPROM_write(        10   ,           0);       // automaticke spousteni vypnute
     EEPROM_write(        11   ,           3);       // 3x prumerovani
//     EEPROM_write_int(    12   ,           0);       // smazat unikatni identifikaci (po resetu se vytvori nova)
     EEPROM_write(        14   ,           1);       // komunikacni rychlost 9600; SLAVE adresa = 1
     EEPROM_write(        15   ,           1);       // testuje kontrolni bajt pri komunikaci pres RS485
     EEPROM_write(        16   ,          20);       // nestabilni jas je hlaseny pri rozdilu dvou sousednich mereni nad 0.2 %
     EEPROM_write(        17   ,           1);       // pocet cidel na expanzni desce
     EEPROM_write(        18   ,           0);       // zachytny bod hledani (po kazdem 100. zaznamu se aktualizuje kvuli rychlejsimu hledani volneho mista v EEPROM)
     EEPROM_write(        19   ,           0);       // hranice plosneho jasu pro ukladani do EEPROM a na kartu v desetinach mag/arcsek2) (0 az 25,5 mag/arcsek2)
     
  }



void chyba(byte kod_chyby)
  {
#ifdef displej_TM1637_5    
    switch(kod_chyby)
      {
        case 1:                        // chyba I2C komunikace (cidlo svetla, EEPROM, expander)
          zobraz_text(18);             //   Err-1
          break;

        case 2:                        // chyba SD karty
          zobraz_text(15);             //   Sd-Er
          break;

        case 3:                        // neni zasunuta SD karta
          zobraz_text(21);             //   no-5d  (No-SD)
          break;

        case 4:                        // Spatny datum v RTC
          zobraz_text(27);             //   Err-3
          break;

        case 5:                        // Chyba v kalibracni tabulce svetla
          zobraz_text(30);             //   Err-4
          break;

        case 6:                        // Chyba v kalibracni tabulce svetla
          zobraz_text(31);             //   Err-5
          break;


        default:
          break;
      }

    for (byte bz = 0; bz < 3 ; bz++ )  // pri chybe se jeste 3x rychle zapipa
      {
        tone(pin_bzuk,600, 10);      
        delay(100);
      }    

    delay(500);
    
    zobraz_text(28);                   // po pul sekunde se zhasne displej
        
    
#endif

#ifdef signal_LED    
    LED_B_onoff(true);                 // rozsviti modrou LED
    delay(500);
#endif
  }
//----------------------------------------------



//----------------------------------------------
// prepinani cidel svetla
//  kdyz je index zadany, prepne se na konkretni cislo
void prepni_cidlo_svetla(byte index)
  {
    if (index <= pocet_cidel_svetla and index > 0)
      {
        index_cidla_svetla = index;
        
        // nacist novou kalibracni tabulku z EEPROM podle aktualne prepnuteho cidla svetla
        unsigned int posun = ((index_cidla_svetla-1) * 60) + 50;   // podle aktualniho indexu cidla svetla se zvoli spravna oblast v EEPROM pro kalibracni tabulku
        for (byte i = 0 ; i < 15 ; i++)
          {
            kal_svetlo[i][0] = EEPROM_read_int(posun   +   (i*4));
            kal_svetlo[i][1] = EEPROM_read_int(posun + 2 + (i*4));
          }
        expa_switch(index_cidla_svetla);                           // prepnout napajeni na prislusne cidlo

      
      }   // pri spatne zadane hodnote se nic nedeje
  }
//----------------------------------------------



//----------------------------------------------
// kdyz parametr zadany neni, prepne se na nasledujici cidlo
void prepni_cidlo_svetla(void)
  {
    index_cidla_svetla ++;
    if (index_cidla_svetla > pocet_cidel_svetla) index_cidla_svetla = 1;
    prepni_cidlo_svetla(index_cidla_svetla);    
  }
//----------------------------------------------



//----------------------------------------------
// pri dlouhych operacich (nebo ve smyckach, ktere bezi mimo hlavni loop) se pravidelne kontroluje, jestli doslo k nejakemu pozadavku o komunikaci,
// nebo jestli uz neni cas provest automaticke spusteni mereni
void interrupty(void)
  {

    test_485_kom();                                                   // testovani, jsetli neprislo neco po seriove lince RS485
    test_ser_kom();                                                   // testovani, jsetli neprislo neco po USB lince

    if (digitalRead(pin_TS) == LOW and blokuj_TS == false)            // pri zkratu pinu SCK a GND na bocnim konektoru desky WeAct (BluePill +) se zaznamena casove razitko
      {
        blokuj_TS = true;                                             // zablokovani dalsiho zapisu casove znacky az do uvolneni tlacitka
        casove_razitko();
      }

    if (digitalRead(pin_TS) == HIGH)
      {
        blokuj_TS = false;                                            // odblokovani dalsiho zapisu casove znacky po uvolneni tlacitka
      }
          
    if (automat > 0)                                                  // kdyz je nastavene automaticke snimani
      {
        if (millis() - posledni_autospusteni > (automat * 60000UL))   // automat udava cas v minutach - to je (60 * 1000ms)
          {
            posledni_autospusteni = millis();  
            mereni(6);                                                // spusteni automatickeho mereni          
          }
      }      
  }
//----------------------------------------------



//----------------------------------------------
// Prepise jen jeden bit v kazdem zaznamu, takze je prostor oznaceny jako prazdny.
// Data jsou ale az do dalsiho prepsani citelna (ve vypisech ze poznaji podle toho, ze misto napisu " SEC" nebo "SELC" maji zobrazeno " _EC", nebo "_ELC")
void soft_format_EEPROM(void)
  {
    Serial.println(lng164);                                           //             Serial.println("SOFT format EEPROM ...");    // bargraf do serioveho terminalu
    Serial.print('|');
    unsigned int pocitadlo = 0;
    for (byte i = 0; i< (((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100) ; i++)
      {
        Serial.print('-');
      }
    Serial.println('|');
    Serial.print('|');

    dokoncene_mereni = 0;                                             // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres RS485    
    for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
      {
        test_485_kom();                                               // test, jestli neprisel nejaky pozadavek po komunikaci RS485   
        pocitadlo ++;
        if ((pocitadlo % 100) == 0)
          {
            Serial.print('#');
            zobraz_cislo((((max_EEPROM-min_EEPROM) / delka_zaznamu) / 100) - (adresa / delka_zaznamu  / 100) , 7);        // vystup odpocitavani na displej podle delky zaznamu se zacina na 45 az 160 (pro 4500 az 16000 zaznamu) 
          }
        byte pomprom = EEPROM_read(adresa);
        if ((pomprom & 0b10000000))                                   // pokud je v EEPROM na zacatku bloku nastaveny bit7 na '1', znamena to obsazeny blok a tato '1' se smaze
          {
            delay(3);
            EEPROM_write(adresa , (pomprom & 0b01111111));            // bit7 se nastavi na '0' 
          }
      }
    EEPROM_write(18 ,0);                                              // zachytny bod pro pristi hledani volneho mista v EEPROM se nastavi na zacatek (adresa 500)

    dokoncene_mereni = 2;                                             // po skonceni formatovani se odblokuje zablokovane spousteni dalsiho mereni pres RS485
    Serial.println('|');
    Serial.println(lng166);                                           //             Serial.println("EEPROM byla zformatovana.");

    delay(1000);
    SOFT_RESET();
    
  }
//----------------------------------------------



//----------------------------------------------
// smaze uplne vsechny datove bunky v EEPROM
void hard_format_EEPROM(void)
  {
    Serial.println(lng165);                                           //             Serial.println("HARD format EEPROM ...");        // bargraf do serioveho terminalu
    Serial.print('|');
    for (byte i = 0; i < 1+( (max_EEPROM-min_EEPROM) / 1000) ; i++)
      {
        Serial.print('-');
      }
    Serial.println('|');
    Serial.print('|');
    
    dokoncene_mereni = 0;                                             // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni   
    for (unsigned long adresa = min_EEPROM; adresa < max_EEPROM ; adresa ++)
      {
         test_485_kom();                                              // test, jestli neprisel nejaky pozadavek po komunikaci RS485   
         if (adresa % 1000 == 0)
           {
             Serial.print('#');
                                                                       // vystup odpocitavani na displej (zacina se od 123 a pri kazdem kilobajtu se odecte 1 ) 
             zobraz_cislo((max_EEPROM / 1000) - ((adresa - min_EEPROM) / 1000) , 5);
           }
         delay(3);
         EEPROM_write(adresa ,0);                                     // smazani bunky
      }
    EEPROM_write(18 ,0);                                              // zachytny bod pro pristi hledani volneho mista v EEPROM se nastavi na zacatek (adresa 500)

    dokoncene_mereni = 2;                                             // po skonceni formatovani se odblokuje zablokovane spousteni dalsiho mereni pres RS485
    Serial.println('|');
    Serial.println(lng166);                                           //             Serial.println("EEPROM byla zformatovana.");
    delay(1000);
    SOFT_RESET();
  
  }
//----------------------------------------------


  
//----------------------------------------------
void vypis_EEPROM(unsigned int parametr)
  {
    boolean prazdno = true;                                           // kdyz nebude ve vypisu ani jeden zaznam, zahlasi se na konci "Zadny ulozeny zaznam"

    if (parametr != 65531)    Serial.println(hlavicka);               // v seriovem terminalu se hlavicka zobrazuje vzdycky (krome funkce LIST z menu)


    switch (parametr)
      {

        case   65534:                                                 // vypis posledniho zaznamu
          {
            priprav_1_zaznam(adr_posl_zaznamu);                       // v promenne 'adr_posl_zaznamu' je ulozena adresa zacatku posledniho zaznamu
            Serial.println(vystupni_retezec);
            prazdno = false;
          }
        break;
        
        
        case  65533:                                                  // vypis vsech zaznamu (zaznamu ktere maji znacku v prvnim bajtu) - nevypisuji se SOFT formatovane zaznamy
          {
            dokoncene_mereni = 0;                                     // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres seriovou linku nebo pres autoomat

            unsigned int pocitadlo_radek = (max_zaznam_EEPROM - min_EEPROM) / delka_zaznamu;
            for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
              {
                if ((EEPROM_read(adresa) & 0b10000000) != 0)          // polozka v EEPROM je obsazena
                  {
                    prazdno = false;
                    priprav_1_zaznam(adresa);
                    Serial.println(vystupni_retezec);
                    test_485_kom();                                   // test, jestli neprisel nejaky pozadavek po komunikaci RS485

                    if (digitalRead(pin_tl_ok) == LOW)                // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                      {
                        delay(20);
                        while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                        break;
                      }
                  }

                pocitadlo_radek --;
                if (pocitadlo_radek % 20 == 0)
                  {
                    zobraz_cislo((pocitadlo_radek / 20) , 6);         // vystup odpocitavani na displej
                  }

              
              }                     
          }
        dokoncene_mereni = 2;                                         // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni

        break;

        
        case  65532:                                                  // vypis cele EEPROM (krome tech zaznamu, ktere maji datum nastaveny na 1.1.1970) - vypisuji se i SOFT formatovane zaznamy
          {
            dokoncene_mereni = 0;                                     // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni pres RS485

            unsigned int pocitadlo_radek = (max_zaznam_EEPROM - min_EEPROM) / delka_zaznamu;            
            for (unsigned long adresa = min_EEPROM; adresa <= max_zaznam_EEPROM ; adresa = adresa + delka_zaznamu)
              {
                if (EEPROM_read_long(adresa) > 0)                     // vypisuji se jen zaznamy, ktere maji nejaky normalni datum
                  {
                    prazdno = false;
                    priprav_1_zaznam(adresa);
                    Serial.println(vystupni_retezec);
                    test_485_kom();                                   // test, jestli neprisel nejaky pozadavek po komunikaci RS485

                    if (digitalRead(pin_tl_ok) == LOW)                // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                      {
                        delay(20);
                        while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                        break;
                      }
                      

                    pocitadlo_radek --;
                    if (pocitadlo_radek % 20 == 0)
                      {
                        zobraz_cislo((pocitadlo_radek / 20) , 6);     // vystup odpocitavani na displej
                      }                    
                  }
              }                     
          }
        dokoncene_mereni = 2;                                         // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni pres RS485
        break;

        case   65531:                                                 // vypis stareho zaznamu na displej
          {
            //---------------
            boolean vyskocit_ze_zobrazeni = false;
            
            while (vyskocit_ze_zobrazeni == false)
              {

                // nalezeni adresy pred 'list' zaznamy
                unsigned long adr_100 = adr_posl_zaznamu;
                
                int pocet = list;

                while (pocet > 0)                                     // nekolikrat ('list' krat) se odecte delka zaznamu ...
                  {
                    pocet --;
                    adr_100 = adr_100 - delka_zaznamu;
                    if (adr_100 < min_EEPROM)                         //  ... kdyz behem toho dojde k podteceni pod zacatek oblasti s daty ...
                      {
                        adr_100 = max_zaznam_EEPROM;                  // ... nastavi se adresa na posledni pouzitelny zaznam v EEPROM
                      }
                  }

                if (EEPROM_read(adr_100) & 0b10000000)                // v EEPROM se nachazi platny zaznam
                  {
                    priprav_1_zaznam(adr_100);                

                    if (EEPROM_read(adr_100) & 0b01000000)            // specialni pripad zaznamu pro funkci stopky
                      {
                        switch (EEPROM_read(adr_100 + 7))
                          {
                            case 1:                                                                // START
                              {
                                pom_disp_scroll[4]  = 109;                                         // S
                                pom_disp_scroll[5]  = 120;                                         // t
                                pom_disp_scroll[6]  = 119;                                         // A
                                pom_disp_scroll[7]  =  80;                                         // r
                                pom_disp_scroll[8]  = 120;                                         // t
                                pom_disp_scroll[9]  =   0;                                         // ' '
                                pom_disp_scroll[10] =   graf_def[vystupni_retezec[31]-48];         // desitky dni
                                pom_disp_scroll[11] =   graf_def[vystupni_retezec[32]-48] + 128;   // jednotky dni + tecka
                                pom_disp_scroll[12] =   graf_def[vystupni_retezec[34]-48];         // desitky mesicu
                                pom_disp_scroll[13] =   graf_def[vystupni_retezec[35]-48] + 128;   // jednotky mesicu + tecka
                                pom_disp_scroll[14] =   0;    // ' '
                                pom_disp_scroll[15] =   graf_def[vystupni_retezec[44]-48];         // desitky hodin
                                pom_disp_scroll[16] =   graf_def[vystupni_retezec[45]-48];         // jednotky hodin
                                pom_disp_scroll[17] =   64;    // '-'
                                pom_disp_scroll[18] =   graf_def[vystupni_retezec[47]-48];         // desitky minut
                                pom_disp_scroll[19] =   graf_def[vystupni_retezec[48]-48];         // jednotky minut
                               
                                vyskocit_ze_zobrazeni = disp_zaznam_scroll(20);
                                
                              
                              }
                            break;
        
                            case 2:                                                                // STOP
                              {
                                pom_disp_scroll[4]  = 109;                                         // S
                                pom_disp_scroll[5]  = 120;                                         // t
                                pom_disp_scroll[6]  =  92;                                         // o
                                pom_disp_scroll[7]  = 115;                                         // P
                                pom_disp_scroll[8]  =   0;                                         // ' '
                                pom_disp_scroll[9]  =   graf_def[vystupni_retezec[32]-48];         // desitky hodin
                                pom_disp_scroll[10] =   graf_def[vystupni_retezec[33]-48];         // jednotky hodin
                                pom_disp_scroll[11] =   64;    // '-'
                                pom_disp_scroll[12] =   graf_def[vystupni_retezec[35]-48];         // desitky minut
                                pom_disp_scroll[13] =   graf_def[vystupni_retezec[36]-48];         // jednotky minut
                                pom_disp_scroll[14] =   64;    // '-'
                                pom_disp_scroll[15] =   graf_def[vystupni_retezec[38]-48];         // desitky sekund
                                pom_disp_scroll[16] =   graf_def[vystupni_retezec[39]-48] + 128;   // jednotky sekund + tecka
                                pom_disp_scroll[17] =   graf_def[vystupni_retezec[48]-48];         // desetiny sekund
                                pom_disp_scroll[18] =   graf_def[vystupni_retezec[49]-48];         // setiny sekund
                                pom_disp_scroll[19] =   graf_def[vystupni_retezec[50]-48];         // tisiciny sekund
                                
                                vyskocit_ze_zobrazeni = disp_zaznam_scroll(20);
        
                              
                              }
                            break;
        
                            case 3:                                                                // LAP
                              {
                                pom_disp_scroll[4]  =  56;                                         // L
                                pom_disp_scroll[5]  = 119;                                         // A
                                pom_disp_scroll[6]  = 115;                                         // P
                                pom_disp_scroll[7]  =   0;                                         // ' '
                                pom_disp_scroll[8]  =   graf_def[vystupni_retezec[89]-48];         // desitky z pocitadla mezicasu
                                pom_disp_scroll[9]  =   graf_def[vystupni_retezec[90]-48];         // jednotky z pocitadla mezicasu
                                pom_disp_scroll[10] =   0;    // ' '                        
                                pom_disp_scroll[11] =   graf_def[vystupni_retezec[32]-48];         // desitky hodin
                                pom_disp_scroll[12] =   graf_def[vystupni_retezec[33]-48];         // jednotky hodin
                                pom_disp_scroll[13] =   64;    // '-'
                                pom_disp_scroll[14] =   graf_def[vystupni_retezec[35]-48];         // desitky minut
                                pom_disp_scroll[15] =   graf_def[vystupni_retezec[36]-48];         // jednotky minut
                                pom_disp_scroll[16] =   64;    // '-'
                                pom_disp_scroll[17] =   graf_def[vystupni_retezec[38]-48];         // desitky sekund
                                pom_disp_scroll[18] =   graf_def[vystupni_retezec[39]-48] + 128;   // jednotky sekund + tecka
                                pom_disp_scroll[19] =   graf_def[vystupni_retezec[48]-48];         // desetiny sekund
                                pom_disp_scroll[20] =   graf_def[vystupni_retezec[49]-48];         // setiny sekund
                                pom_disp_scroll[21] =   graf_def[vystupni_retezec[50]-48];         // tisiciny sekund
        
                                vyskocit_ze_zobrazeni = disp_zaznam_scroll(22);
                              }
                            break;
        
                            case 4:                                                                // CONT
                              {
                                pom_disp_scroll[4]  =  57;                                         // C
                                pom_disp_scroll[5]  =  92;                                         // o
                                pom_disp_scroll[6]  =  84;                                         // n
                                pom_disp_scroll[7]  = 120;                                         // t
                                pom_disp_scroll[8]  =   0;                                         // ' '
                                pom_disp_scroll[9]  =   graf_def[vystupni_retezec[31]-48];         // desitky dni
                                pom_disp_scroll[10] =   graf_def[vystupni_retezec[32]-48] + 128;   // jednotky dni + tecka
                                pom_disp_scroll[11] =   graf_def[vystupni_retezec[34]-48];         // desitky mesicu
                                pom_disp_scroll[12] =   graf_def[vystupni_retezec[35]-48] + 128;   // jednotky mesicu + tecka
                                pom_disp_scroll[13] =   0;    // ' '
                                pom_disp_scroll[14] =   graf_def[vystupni_retezec[44]-48];         // desitky hodin
                                pom_disp_scroll[15] =   graf_def[vystupni_retezec[45]-48];         // jednotky hodin
                                pom_disp_scroll[16] =   64;    // '-'
                                pom_disp_scroll[17] =   graf_def[vystupni_retezec[47]-48];         // desitky minut
                                pom_disp_scroll[18] =   graf_def[vystupni_retezec[48]-48];         // jednotky minut
        
                                vyskocit_ze_zobrazeni = disp_zaznam_scroll(19);
                              }
                            break;
        
                            
                          }
                      }
    
                    //----                  
                    if (EEPROM_read(adr_100) & 0b00100000)                                         // specialni pripad zaznamu pro funkci casoveho razitka
                      {
                        pom_disp_scroll[4]  = 120;                                                 // t
                        pom_disp_scroll[5]  =   4;                                                 // i
                        pom_disp_scroll[6]  =  55;                                                 // M
                        pom_disp_scroll[7]  = 121;                                                 // E                    
                        pom_disp_scroll[8]  =   0;                                                 // ' '
                        pom_disp_scroll[9]  =   graf_def[vystupni_retezec[69]-48];                 // stovky z poradoveho cisla razitka
                        pom_disp_scroll[10] =   graf_def[vystupni_retezec[70]-48];                 // desitky  z poradoveho cisla razitka
                        pom_disp_scroll[11] =   graf_def[vystupni_retezec[71]-48];                 // jednotky z poradoveho cisla razitka
                        pom_disp_scroll[12] =   0;    // ' '
                        pom_disp_scroll[13] =   graf_def[vystupni_retezec[31]-48];                 // desitky dni
                        pom_disp_scroll[14] =   graf_def[vystupni_retezec[32]-48] +128;            // jednotky dni s teckou
                        pom_disp_scroll[15] =   graf_def[vystupni_retezec[34]-48];                 // desitky mesicu
                        pom_disp_scroll[16] =   graf_def[vystupni_retezec[35]-48] +128;            // jednotky mesicu s teckou
                        pom_disp_scroll[17] =   0;    // ' '
                        pom_disp_scroll[18] =   graf_def[vystupni_retezec[44]-48];                 // desitky hodin
                        pom_disp_scroll[19] =   graf_def[vystupni_retezec[45]-48];                 // jednotky hodin
                        pom_disp_scroll[20] =   64;    // '-'
                        pom_disp_scroll[21] =   graf_def[vystupni_retezec[47]-48];                 // desitky minut
                        pom_disp_scroll[22] =   graf_def[vystupni_retezec[48]-48];                 // jednotky minut
                        pom_disp_scroll[23] =   64;    // '-'
                        pom_disp_scroll[24] =   graf_def[vystupni_retezec[50]-48];                 // desitky sekund
                        pom_disp_scroll[25] =   graf_def[vystupni_retezec[51]-48];                 // jednotky sekund
    
                        vyskocit_ze_zobrazeni = disp_zaznam_scroll(26);
                        
                      }
    
    
                    //----                  
                    if ((EEPROM_read(adr_100) & 0b01100000) == 0)                                  // bezny zaznam svetla (nebo kalibracni zaznam)
                      {
                        pom_disp_scroll[4]  =  znacka_listu;                                       // 1 znak z poznamky (A=automat, C=kalibrace, U=tlacitko Up ....)
                        pom_disp_scroll[5]  =  0;                                                  // mezera
                        
                        pom_disp_scroll[6]  =  graf_def[vystupni_retezec[31]-48];                  // desitky dni
                        pom_disp_scroll[7]  =  graf_def[vystupni_retezec[32]-48] | 0b10000000;     // jednotky dni s teckou
                        pom_disp_scroll[8]  =  graf_def[vystupni_retezec[34]-48];                  // desitky mesicu
                        pom_disp_scroll[9]  =  graf_def[vystupni_retezec[35]-48] | 0b10000000;     // jednotky mesicu s teckou
                        pom_disp_scroll[10] =  0;                                                  // mezera
                        pom_disp_scroll[11] =  graf_def[vystupni_retezec[44]-48];                  // desitky hodin
                        pom_disp_scroll[12] =  graf_def[vystupni_retezec[45]-48];                  // jednotky hodin
                        pom_disp_scroll[13] =  64;                                                 // mezi hodinami a minutami je pomlcka
                        pom_disp_scroll[14] =  graf_def[vystupni_retezec[47]-48];                  // desitky minut
                        pom_disp_scroll[15] =  graf_def[vystupni_retezec[48]-48];                  // jednotky minut
                        pom_disp_scroll[16] =  0;                                                  // mezera
                        if (vystupni_retezec[75] == '0')   pom_disp_scroll[17] = 0;                // nula na radu desitek mag se nezobrazuje
                        else       pom_disp_scroll[17] = graf_def[vystupni_retezec[75]-48];        // kdyz jsou nejake desitky mag, tak se zobrazi
                        pom_disp_scroll[18] =  graf_def[vystupni_retezec[76]-48] | 0b10000000;     // jednotky mag s teckou
                        pom_disp_scroll[19] =  graf_def[vystupni_retezec[78]-48];                  // desetiny mag
                        pom_disp_scroll[20] =  graf_def[vystupni_retezec[79]-48];                  // setiny mag
                        pom_disp_scroll[21] =  graf_def[vystupni_retezec[80]-48];                  // tisiciny mag
                                        
                        vyskocit_ze_zobrazeni = disp_zaznam_scroll(22);
                      }
                  
                  }
                else                          // v EEPROM neni platny zaznam (bud je tam prazdna oblst, nebo je tam nejaky drive smazany zanam - SOFTformatovany)
                  {
                    // najeti 5 pomlcek na displej
                    pom_disp_scroll[4]  = 64;                                                      // -
                    pom_disp_scroll[5]  = 64;                                                      // -
                    pom_disp_scroll[6]  = 64;                                                      // -
                    pom_disp_scroll[7]  = 64;                                                      // -
                    pom_disp_scroll[8]  = 64;                                                      // -
                    vyskocit_ze_zobrazeni = disp_zaznam_scroll(9);
                  }
    
    
                byte pompam4 = D_pamet[4];                                                         // zapamatuje se stav displeje pro nasledujici smycku, ve ktere muze dojit k jeho zhasnuti
                byte pompam3 = D_pamet[3];                                                         //    pri aktivaci interruptu (automat, seriova komunikace)
                byte pompam2 = D_pamet[2];
                byte pompam1 = D_pamet[1];
                byte pompam0 = D_pamet[0];
    
                
                byte pom_pocitadlo = 0;

                // dokud neni stisknuto zadne tlacitko, testuje se komunikace a na displeji je zobrazen konec posledniho rotovaneho udaje
                while (digitalRead(pin_tl_ok) == HIGH and digitalRead(pin_tl_up) == HIGH and digitalRead(pin_tl_dn) == HIGH )
                  {                                
                    interrupty();                                                                  // odskoceni na test, jestli neni nejaky pozadavek o komunikaci, nebo jestli neni cas spustit automaticke mereni
    
                    D_pamet[0] = pompam0;                                                          // po navratu z interruptu zobrazi puvodni stav displeje
                    D_pamet[1] = pompam1;
                    D_pamet[2] = pompam2;
                    D_pamet[3] = pompam3;
                    D_pamet[4] = pompam4;
                    aktualizuj_displej();
                    
                    delay(100);
                  }

                // neco bylo stisknuto:
                if (digitalRead(pin_tl_ok) == LOW)                                                 // stisknuto bylo tlacitko OK
                  {
                    pom_pocitadlo = 0;
                    while (digitalRead(pin_tl_ok) == LOW)                                          // dokud se tlacitko OK neuvolni ...
                      {
                        delay(20);                                                                 // ... cekej
                        pom_pocitadlo ++;
                        if (pom_pocitadlo == 50) zobraz_text(28);                                  // prazdny displej v okamziku, kdy je dlouhym stiskem aktivovane opakovani rotace
                        if (pom_pocitadlo > 100) pom_pocitadlo = 100;
                      }
                    delay(20);                                                                     // odruseni zakmitu pri uvolnovani tlacitka
                    prazdno = false;
    
                    if (pom_pocitadlo < 50)                                                        // OK bylo stisknuto na delsi dobu nez sekundu ...
                      {
                        vyskocit_ze_zobrazeni = true;                                              // nastavi se znacka, ze se ma provest navrat do menu
                      }
                    
                  }
                    
    

                if (digitalRead(pin_tl_dn) == LOW)                                                 // stisknuto bylo tlacitko DOLU
                  {
                    list ++;                                                                       // posunuti na predchozi zaznam (starsi)
                  }                                                                                //    promenna 'list' ukazuje, kolikaty zaznam do zpatku se ma zobrazit

                if (digitalRead(pin_tl_up) == LOW)                                                 // stisknuto bylo tlacitko NAHORU
                  {
                    list --;                                                                       // posunuti na dalsi zaznam (cerstvejsi)
                    if (list < 0)                                                                  // tady musi byt kontrala na podlezeni 0 (neda se zobrazit zaznam, ktery jeste nebyl zaznamenany)
                      {
                        list = 0;                           
                        tone(pin_bzuk,800, 30);                                                    // chybove pipnuti po dosazeno minima (posledni zaznam) - novejsi zaznam uz neni
                      }
                  }

                
              }

            zobraz_text(28);                                                                       // prazdny displej
          }
        break;


        case  65535:                                                                               // vypise poslednich 100 zaznamu (s testem na prechod pres vrchol pouzitelne EEPROM) 
          {
            byte pocet = 100;
            // nalezeni adresy pred 100 zaznamy
            unsigned long adr_100 = adr_dalsiho_zaznamu;

            while (pocet > 0)                                                                      // 100x se odecte delka zaznamu ...
              {
                pocet --;
                adr_100 = adr_100 - delka_zaznamu;
                if (adr_100 < min_EEPROM)                                                          //  ... kdyz behem toho dojde k podteceni pod zacatek oblasti s daty ...
                  {
                    adr_100 = max_zaznam_EEPROM;                                                   // ... nastavi se adresa na posledni pouzitelny zaznam v EEPROM
                  }
              }
            
            pocet = 100;
            dokoncene_mereni = 0;                                                                  // je to dlouha operace, behem ktere je blokovano spusteni dalsiho mereni

            while (pocet > 0)
              {
                test_485_kom();                                                                    // test, jestli neprisel nejaky pozadavek po komunikaci RS485                
                pocet --;
                adr_100 = adr_100 + delka_zaznamu;
                if (adr_100 > max_zaznam_EEPROM)
                  {
                    adr_100 = min_EEPROM;
                  }
                if ((EEPROM_read(adr_100) & 0b10000000) != 0)                                      // polozka v EEPROM je obsazena
                  {
                    prazdno = false;
                    priprav_1_zaznam(adr_100);
                    Serial.println(vystupni_retezec);

                    if (digitalRead(pin_tl_ok) == LOW)                                             // predcasne ukonceni dlouheho vypisu stiskem tlacitka OK
                      {
                        delay(20);
                        while (digitalRead(pin_tl_ok) == LOW)    delay(20);
                        break;
                      }
                    zobraz_cislo( pocet , 6);                                                      // zbyvajici pocet radek na displej
                  }

              }                    
          }

        dokoncene_mereni = 2;                                                                      // po skonceni vypisu se odblokuje zablokovane spousteni dalsiho mereni pres RS485
        break;
        
        case  0:                                                                                   // mimo rozsah
          prazdno = false;                                                                         //  na konci nevypisuje hlaseni "Zadny ulozeny zaznam" (ani v pripade, ze by opravdu zadny zaznam na zvolene polozce neexistoval)
          Serial.print(lng167);                                                                    //             Serial.print("Zvolena polozka je mimo rozsah EEPROM (1 - ");
          Serial.print((max_EEPROM - min_EEPROM) / delka_zaznamu);
          Serial.println(')');
        break;


        default:                                                                                   // jakykoliv jiny parametr znamena vypis jedne zvolene polozky
          {
            prazdno = false;                                                                       // pri vypisu zvolene polozky se na konci nevypisuje hlaseni "Zadny ulozeny zaznam"
            unsigned long adresa = ((parametr-1) * delka_zaznamu) + min_EEPROM;                    //       (ani v pripade, ze by opravdu zadny zaznam na zvolene polozce neexistoval)      
            if (adresa <= max_zaznam_EEPROM)
              {
                priprav_1_zaznam(adresa);
                Serial.println(vystupni_retezec);
              }
            else
              {
                Serial.print(lng167);                                                              //             Serial.print("Zvolena polozka je mimo rozsah EEPROM (1 - ");
                Serial.print((max_EEPROM - min_EEPROM) / delka_zaznamu);
                Serial.println(')');
              }
          }
        
        
      }
      
    if (prazdno == true)
      {
        Serial.println(lng168);                                                                    //             Serial.println("Zadny ulozeny zaznam");   
      }
  }
//----------------------------------------------



//----------------------------------------------
// do docasneho pole EEPROM posklada namerene hodnoty a aktualni cas
//  vsechny potrebne hodnoty si bere  z globalnich promennych:
//       selc, int_svetlo, slave_addr, int_infra, int_full, byte_cnf, int_teplota, int_vlhkost a int_tlak
//  vystupem je globalni 'pole_EEPROM[]'
void priprava_pole_EEPROM(void)
  {
                                                                       // ulozeni merenych hodnot do pole 
    pole_EEPROM[0] =  0b10000000 + selc;                               // znacka, ze je blok adres v EEPROM obsazeny (bit7='1') jestli je SEC(bit0='0'), nebo SELC(bit0='1')

    pole_EEPROM[0] = pole_EEPROM[0] | (index_cidla_svetla << 1);       // pro rozsirujici desku s vice cidly svetla se uklada do prvniho bajtu i index cidla svetla 1 az 7

    unsigned long tt = rtc_korekce();                                  // zjisteni aktualniho casu v RTC (v GMT)

    pole_EEPROM[1] =  ((unsigned long)tt >> 24) & 0xFFUL;              // 4 bajty pro pocet sekund od 1.1.1970 (s korekci nepresnosti a pripadnym prevodem na letni cas)
    pole_EEPROM[2] =  ((unsigned long)tt >> 16) & 0xFFUL;
    pole_EEPROM[3] =  ((unsigned long)tt >>  8) & 0xFFUL;
    pole_EEPROM[4] =  ((unsigned long)tt      ) & 0xFFUL;    

    pole_EEPROM[5] = (int_svetlo >> 8) & 0xFF;
    pole_EEPROM[6] = (int_svetlo     ) & 0xFF;
    
    pole_EEPROM[7] = (slave_addr << 4) + znacka ;                      // SLAVE adresa se uklada do hornich 4 bitu, spodni 1 bit pro stabilni jas; 3 bity jsou znacky (tlacitko,kalibrace,rucni spousteni, automat)

    byte indexpole = 8;

#ifdef ukladat_infra      
    pole_EEPROM[indexpole]   = (int_infra >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_infra     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_full       
    pole_EEPROM[indexpole]   = (int_full >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_full     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_contr_reg 
    pole_EEPROM[indexpole] = byte_cnf;
    indexpole = indexpole + 1;
#endif

#ifdef ukladat_teplotu    
    pole_EEPROM[indexpole]   = (int_teplota >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_teplota     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_vlhkost    
    pole_EEPROM[indexpole]   = (int_vlhkost >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_vlhkost     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_tlak       
    pole_EEPROM[indexpole]   = (int_tlak >> 8) & 0xFF;
    pole_EEPROM[indexpole+1] = (int_tlak     ) & 0xFF;
    indexpole = indexpole + 2;
#endif

#ifdef ukladat_GPS
    pole_EEPROM[indexpole]   = (GPS_lat >> 24) & 0xFF;
    pole_EEPROM[indexpole+1] = (GPS_lat >> 16) & 0xFF;
    pole_EEPROM[indexpole+2] = (GPS_lat >>  8) & 0xFF;
    pole_EEPROM[indexpole+3] = (GPS_lat      ) & 0xFF;
    pole_EEPROM[indexpole+4] = (GPS_lon >> 24) & 0xFF;
    pole_EEPROM[indexpole+5] = (GPS_lon >> 16) & 0xFF;
    pole_EEPROM[indexpole+6] = (GPS_lon >>  8) & 0xFF;
    pole_EEPROM[indexpole+7] = (GPS_lon      ) & 0xFF;
    pole_EEPROM[indexpole+8] = (GPS_alt >>  8) & 0xFF;
    pole_EEPROM[indexpole+9] = (GPS_alt      ) & 0xFF;

    indexpole = indexpole + 10;
#endif    
  }
//----------------------------------------------



//----------------------------------
// reset zpusobeny zamernym pretecenim watchdogu
void SOFT_RESET()
  {
    zobraz_text(22);                                                                   //  "rE5Et" 
    delay(1000);
    iwdg_init(IWDG_PRE_256, 10);                                                       // nastaveni WD na 50 milisekund  (10*5)
    iwdg_feed();                                                                       // a okamzite obcerstveni WD
    delay(1000);                                                                       // behem pauzy dojde k aktivaci WD a resetu procesoru 
    
  }
//----------------------------------------------



//----------------------------------------------
// jednoducha cervena svitilna s regulaci jasu v 5 stupnich 
void svitilna(void)
  {
    zobraz_text(29);                                                                   // pomlcky na displeji "-----" 
    while (digitalRead(pin_tl_ok) == LOW)                                              // do teto funkce se vlezlo stiskem tlacitka OK, tak se musi pockat, az se uvolni
      {
        delay(20);
      }
    delay(20);

    byte pamet_jasu = jas_displeje;
    jas_displeje = 7;
    int uroven_jasu = pamet_LED_svitilna;
    boolean lampa_onoff = true;
    boolean konec_lampy = false;
    while (konec_lampy == false)            
      {
        if (lampa_onoff == true)                                                       // lampa je rozsvicena
          {
            if (digitalRead(pin_tl_up) == LOW)                                         // kdyz je rozsviceno, tak se pomoci tlacitek nahoru a dolu muze prenastavit jas
              {
                uroven_jasu ++;
                if (uroven_jasu > 5) uroven_jasu = 5;
              }

            if (digitalRead(pin_tl_dn) == LOW)
              {
                uroven_jasu --;
                if (uroven_jasu < 0) uroven_jasu = 0;
              }

            while((digitalRead(pin_tl_up) == LOW) or (digitalRead(pin_tl_dn) == LOW))  // ceka se, az budou obe tlacitka uvolnena
              {
                delay(20); 
              }

            delay(20);                                                                 // po uvolneni tlacitek se aktualizuje displej na novou uroven jasu
            for (byte segmentovka = 0; segmentovka < 5; segmentovka ++)
              {
                if (segmentovka < uroven_jasu )  D_pamet[segmentovka] = 255;
                else                             D_pamet[segmentovka] = 0;
              }
            aktualizuj_displej();            
          }
        else                                                                           // lampa je zhasnuta
          {
            for (byte segmentovka = 0; segmentovka < 5; segmentovka ++)                // aktualizuj displej na zhasnuty stav
              {
                D_pamet[segmentovka] = 0;
              }
            aktualizuj_displej();
          }

        if (digitalRead(pin_tl_ok) == LOW)                                             // je stisknuto tlacitko OK
          {
            delay(20);
            stisk_time = millis();
            while(digitalRead(pin_tl_ok) == LOW)
              {
                delay(20);
                if (millis() - stisk_time > 500)                                       // OK bylo stisknuto na delsi dobu nez pul sekundy
                  {
                    zobraz_text(29);                                                   // zobrazeni pomlcek
                    pamet_LED_svitilna = uroven_jasu;                                  // pred ukoncenim se zaznamena posledni nastavena hodnota
                    konec_lampy = true;                                                // lampa se ukoncuje.
                  }
              }                                                                        // tlacitko OK bylo uvolneno
            delay(20);
            if(lampa_onoff == true) lampa_onoff = false;                               // lampa se prepne ON / OFF
            else                    lampa_onoff = true;
          }
      
      }

    jas_displeje = pamet_jasu;                                                         // navratem z funkce lampy se vraci puvodni jas displeje
    
  }
//----------------------------------------------



//----------------------------------------------
//  test baterie (blikani desetinnych tecek podle stavu baterie)
//  v nekterych pripadech je prostredni segmentovka pouzita pro signalizaci specialnich rezimu
void test_baterie(byte prostredni_segment)
  {
    #ifdef signal_LED                                          // pri signalizaci pomoci LED se testuje baterie jen ve 2 urovnich
        if (millis()/100 % 20 == 1)
          {
            if (analogRead(pin_kompar_bat) < bat_level_2)      // napeti na baterii je nizke ...
              {
                blik_Y_kratky();                               // ... blika zluta LED
              }
            else                                               // napeti na baterii je v poradku ...
              {
                blik_R();                                      // ... blika cervena LED
              }
          }
      
    #else                                                      // pri signalizaci pomoci displeje se testuje baterie v 5 urovnich 
        if (millis()/200 % 5 == 1)
          {
            D_pamet[4] =   0;                                  // Nejdriv se 4 desetinne tecky pripravi na rozsviceni (jako by byla vybita baterie)
            D_pamet[3] = 128;                                  //    (je to jen priprava - skutecne rozsviceni nastane az na konci tohoto bloku funkci 'aktualizuj_displej')
            D_pamet[2] = 128;
            D_pamet[1] = 128;
            D_pamet[0] = 128;                                  // tecka na radu jednotek blika vzdycky - nezavisle na stavu baterie
    
            // ostatni LED se podle stavu baterie mohou pripravit do zhasnuteho stavu
            unsigned int napeti_baterie = analogRead(pin_AD_BAT);
            if (napeti_baterie > bat_level_3) D_pamet[1] = 0;  // napeti na baterii je vyssi nez LEVEL 3, tecka na radu desitek se nerozsviti (vic nez 7V)
            if (napeti_baterie > bat_level_2) D_pamet[2] = 0;  // napeti na baterii je vyssi nez LEVEL 1, tecka na radu stovek se nerozsviti  (vic nez 6V)
            if (napeti_baterie > bat_level_1) D_pamet[3] = 0;  // napeti na baterii je vyssi nez LEVEL 1, tecka na radu tisicu se nerozsviti  (vic nez 5V)
                                                               // pri napeti mensim nez LEVEL_1 (mene nez 5V) zustavaji svitit vsechny 4 tecky (od radu jednotek do radu tisicu)
    #ifdef verzePCB2
            unsigned int napeti_ref = analogRead(pin_refu);    // pro druhou verzi plosnaku byl zaveden test referencniho napeti
            if (napeti_ref > ref_level)                        // kdyz se digity na referencnim pinu zacnou zvysovat, znamena to, ze klesa referencni (naajeci) napeti pro ADC
              {
                D_pamet[4] = 128;                              // pak se rozblikaji vsechny LED, nezavisle na tom, jaka je zmerena hodnota na pinu AD_BAT    
                D_pamet[3] = 128;
                D_pamet[2] = 128;
                D_pamet[1] = 128;            
              }
    #endif

            D_pamet[2] = D_pamet[2] + prostredni_segment;      // v nekterych rezimech blika prostredni segment na prostredni jednotce
            aktualizuj_displej();                              // Skutecne rozsviceni tecek, ktere zustaly kvuli nizkemu napeti na baterii pripravene k rozsviceni
    
          }
        if (millis()/200 % 5 == 2)                             // po nejakem case vsechny tecky zhasnou
          {
            D_pamet[0] = 0;
            D_pamet[1] = 0;
            D_pamet[2] = 0;
            D_pamet[3] = 0;
            D_pamet[4] = 0;
    
            aktualizuj_displej();      
          }
    #endif
  }
//----------------------------------------------
